@@ -1,11 +1,11 @@
 /* App.m
  * Application class of Cenon
  *
- * Copyright (C) 1995-2005 by vhf interservice GmbH
+ * Copyright (C) 1995-2009 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1995-08-10
- * modified: 2005-11-19
+ * modified: 2009-02-25 (-listFromFile: check extensions for any case)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -54,7 +54,7 @@
 @implementation App
 
 /*
- * modified: 2005-07-22
+ * modified: 2006-12-11
  * Initializes the defaults.
  */
 + (void)initialize
@@ -63,11 +63,7 @@
     [registrationDict setObject:@"." forKey:@"NSDecimalSeparator"];
 
     /* General Preferences defaults */
-#ifdef __APPLE__
-    [registrationDict setObject:@"NO"  forKey:@"doCaching"];
-#else
     [registrationDict setObject:@"YES" forKey:@"doCaching"];
-#endif
     [registrationDict setObject:@"0"   forKey:@"unit"];
     [registrationDict setObject:@"NO"  forKey:@"removeBackups"];
     [registrationDict setObject:@"NO"  forKey:@"expertMode"];
@@ -114,9 +110,9 @@
         path = cdir;
     else
         path = (haveOpenedDocument ? [((NSOpenPanel*)[NSOpenPanel openPanel]) directory]
-                                   : vhfHomeDirectory());
+                                   : NSHomeDirectory());
     if (!path)
-        return vhfHomeDirectory();
+        return NSHomeDirectory();
     return path;
 }
 
@@ -128,16 +124,47 @@
     fixedDocument = docu;
 }
 
+- (void)setActiveDocWindow:(DocWindow*)win
+{
+    activeWindowNum = [win windowNumber];
+}
+
 - (Document*)currentDocument
 {
     if (fixedDocument)
         return fixedDocument;
-    else if ( [[self mainWindow] isMemberOfClass:[DocWindow class]] )
+    /* this is unreliable, because a panel may become the main window! */
+    if ( [[self mainWindow] isMemberOfClass:[DocWindow class]] )
     {	id	docu = [(DocWindow*)[self mainWindow] document];
 
         if ([docu isMemberOfClass:[Document class]])
             return docu;
     }
+    {   NSArray *wins = [self windows];
+        int     i, cnt = [wins count];
+
+        for (i=0; i<cnt; i++)
+            if ( [[wins objectAtIndex:i] windowNumber] == activeWindowNum )
+                return [[wins objectAtIndex:i] document];
+    }
+    /* does this really return the window last worked at? Probably we should remember the active document */
+    /*for ( i=[[self windows] count]-1; i >= 0; i-- )
+    {	DocWindow  *win = [[self windows] objectAtIndex:i];
+
+        if ( [win isMemberOfClass:[DocWindow class]] )
+            return [win document];
+    }*/
+    /*{   int wins[10];
+
+        //NSCountWindowsForContext([self context], &nWin);
+        NSWindowListForContext([self context], 10, wins);   // how to obtain the context? What context is that?
+        for ( i=0; i < 10; i++ )
+        {   DocWindow  *win = wins[i];
+
+            if ( [win isMemberOfClass:[DocWindow class]] )
+                return [win document];
+        }
+    }*/
     return nil;
 }
 - (Document*)openedDocument
@@ -145,7 +172,7 @@
     return document;
 }
 
-- documentInWindow:window
+- (Document*)documentInWindow:(NSWindow*)window
 {
     if ([window isMemberOfClass:[DocWindow class]])
     {	id	docu = [(DocWindow*)window document];
@@ -306,14 +333,17 @@
     document = [Document new];
 }
 
+/* modified: 2009-02-25 (fileNameLC)
+ */
 - (id)listFromFile:(NSString*)fileName
-{   NSArray		*list = nil;
-    NSString		*path, *name;
-    NSFileManager	*defaultManager = [NSFileManager defaultManager];
+{   NSArray         *list = nil;
+    NSString        *path, *name;
+    NSFileManager   *defaultManager = [NSFileManager defaultManager];
+    NSString        *fileNameLC = [fileName lowercaseString];
 
     //if ( [fileName rangeOfString:@".dxf" options:NSCaseInsensitiveSearch].length )
     //if ( [fileName compare:@".dxf" options:NSAnchoredSearch|NSBackwardsSearch|NSCaseInsensitiveSearch] )
-    if ( [fileName hasSuffix:@".dxf"] || [fileName hasSuffix:@".DXF"] )
+    if ( [fileNameLC hasSuffix:@".dxf"] )
     {	id	dxfImport = [[DXFImportSub allocWithZone:[self zone]] init];
         NSData	*data = [NSData dataWithContentsOfFile:fileName];	// get data object
 
@@ -321,8 +351,9 @@
         list = [[dxfImport importDXF:data] retain];	// get list of graphic-objects from import
         [dxfImport release];
     }
-    else if ( [fileName hasSuffix:@".hpgl"] || [fileName hasSuffix:@".hgl"] || [fileName hasSuffix:@".plt"] )
-    {	id	hpglImport;
+    else if ( [fileNameLC hasSuffix:@".hpgl"] || [fileNameLC hasSuffix:@".hgl"] ||
+              [fileNameLC hasSuffix:@".plt"] )
+    {	id      hpglImport;
         NSData	*data;
 
         /* load parameter file
@@ -347,7 +378,7 @@
         list = [[hpglImport importHPGL:data] retain];		// get list of graphic-objects from import
         [hpglImport release];
     }
-    else if ( [fileName hasSuffix:@".ger"] || [fileName hasSuffix:@".gerber"] )
+    else if ( [fileNameLC hasSuffix:@".ger"] || [fileNameLC hasSuffix:@".gerber"] )
     {   id	gerberImport;
         NSData	*data;
 
@@ -402,7 +433,7 @@
         list = [[gerberImport importGerber:data] retain];	// get list of graphic-objects from import
         [gerberImport release];
     }
-    else if ( [fileName hasSuffix:@".drl"] || [fileName hasSuffix:@".din"] )
+    else if ( [fileNameLC hasSuffix:@".drl"] || [fileNameLC hasSuffix:@".din"] )
     {   id		dinImport;
         NSData		*data;
         NSString	*devFileName;
@@ -436,16 +467,16 @@
         list = [[DINImportSub layerListFromGraphicList:list] retain];
         [dinImport release];
     }
-    else if ( [fileName hasSuffix:@".font"] || [fileName hasSuffix:@".pfa"] || [fileName hasSuffix:@".pfb"] )
-    {   id		fontObject, type1Import;
+    else if ( [fileNameLC hasSuffix:@".font"] || [fileNameLC hasSuffix:@".pfa"] || [fileNameLC hasSuffix:@".pfb"] )
+    {   id          fontObject, type1Import;
         NSData		*data;
         NSString	*name;
 
-	if ( [fileName hasSuffix:@".pfa"] || [fileName hasSuffix:@".pfb"] )	// pfa file
+        if ( [fileNameLC hasSuffix:@".pfa"] || [fileNameLC hasSuffix:@".pfb"] )	// pfa file
             name = [NSString stringWithFormat:@"%@", fileName];
-	else
+        else
             name = [fileName stringByAppendingPathComponent:
-                [[fileName lastPathComponent] stringByDeletingPathExtension]];
+        [[fileName lastPathComponent] stringByDeletingPathExtension]];
         type1Import = [[Type1ImportSub allocWithZone:[self zone]] init];	// get new import-object
         data = [NSData dataWithContentsOfFile:name];				// get file-data
 
@@ -455,12 +486,12 @@
     }
 
     /* PostScript, AI */
-    else if ( [fileName hasSuffix:@".eps"] || [fileName hasSuffix:@".ps"] ||
-              [fileName hasSuffix:@".ai"] )
+    else if ( [fileNameLC hasSuffix:@".eps"] || [fileNameLC hasSuffix:@".ps"] ||
+              [fileNameLC hasSuffix:@".ai"] )
         list = [[self listFromPSFile:fileName] retain];
 
     /* PDF */
-    else if ( [fileName hasSuffix:@".pdf"] || [fileName hasSuffix:@".PDF"])
+    else if ( [fileNameLC hasSuffix:@".pdf"] || [fileNameLC hasSuffix:@".PDF"])
     {   PSImportSub	*psImport = [[PSImportSub allocWithZone:[self zone]] init];
 
         [psImport preferArcs:Prefs_PSPreferArcs];
@@ -581,26 +612,26 @@ typedef enum
     IMPORTTO_EXISTINGLAYERS = 2
 } ImportToLayerSelection;
 - (void)import:sender
-{   NSArray		*fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
+{   NSArray     *fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
                                               @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
                                               @"cenon", @"tiff", @"jpg", @"gif",
                                               @"font", @"pfa", @"din", @"drl", nil];
-    NSString		*fileName;
-    id			openPanel = [NSOpenPanel openPanel];
-    static NSString	*openDir = @"";
+    NSString    *fileName;
+    id          openPanel = [NSOpenPanel openPanel];
+    static      NSString	*openDir = @"";
 
     [openPanel setAccessoryView:[importAccessory retain]];
     [openPanel setAllowsMultipleSelection:NO];
 
     if ( [openPanel runModalForDirectory:openDir file:@"" types:fileTypes] )
     {   ImportToLayerSelection	importTo = [iaPopup indexOfSelectedItem];
-        id			list;
-        DocView			*view = [[self currentDocument] documentView];
+        id      list;
+        DocView *view = [[self currentDocument] documentView];
 
         [openDir release];
         openDir = [[[openPanel filename] stringByDeletingLastPathComponent] retain];
 
-	fileName = [openPanel filename];
+        fileName = [openPanel filename];
         list = [self listFromFile:fileName];
 
         switch (importTo)
@@ -668,14 +699,14 @@ typedef enum
  * and loads the specified file into it.
  */
 - (void)openDocument:sender
-{   NSArray		*fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
+{   NSArray         *fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
                                               @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
                                               @"cenon", @"tiff", @"jpg", @"gif",
                                               @"font", @"pfa", @"din", @"drl", nil];
-    NSArray		*fileNames;
-    id			openPanel = [NSOpenPanel openPanel];
-    int			i, cnt;
-    static NSString	*openDir = @"";
+    NSArray         *fileNames;
+    id              openPanel = [NSOpenPanel openPanel];
+    int             i, cnt;
+    static NSString *openDir = @"";
 
     [openPanel setAllowsMultipleSelection:YES];
     if ( [openPanel runModalForDirectory:openDir file:@"" types:fileTypes] )
@@ -683,10 +714,10 @@ typedef enum
         [openDir release];
         openDir = [[[openPanel filename] stringByDeletingLastPathComponent] retain];
 
-	fileNames = [openPanel filenames];
+        fileNames = [openPanel filenames];
         cnt = [fileNames count];
-	for ( i=0; i<cnt; i++ )
-	    haveOpenedDocument = [self openFile:[fileNames objectAtIndex:i]] || haveOpenedDocument;
+        for ( i=0; i<cnt; i++ )
+            haveOpenedDocument = [self openFile:[fileNames objectAtIndex:i]] || haveOpenedDocument;
     }
 }
 
@@ -828,7 +859,15 @@ typedef enum
             NSLog(@"Cannot load Info interface file");
     }
 
-    [serialNumber setStringValue:@""];
+    [serialNumber  setStringValue:@""];
+
+    /*{   NSDictionary    *infoDict = [[NSBundle mainBundle] infoDictionary];
+        NSString    *version;
+
+        //NSLog(@"%@", infoDict);
+        if ( (version=[infoDict objectForKey:@"CFBundleVersion"]) )
+            [infoVersionNo setStringValue:version]; // ex: 3.9.0 pre 3 (2008-03-01)
+    }*/
 
     [[NSNotificationCenter defaultCenter] postNotificationName:InfoPanelWillDisplay
                                                         object:nil userInfo:nil];
@@ -862,16 +901,49 @@ typedef enum
     return preferencesPanel;
 }
 
-#if 0
+/* load PDF documentation
+ * Note: each module can add a help menu entry to load it's docu
+ * modified: 2007-07-22
+ */
+#if defined(GNUSTEP_BASE_VERSION) || defined(__APPLE__)	// GNUstep or Apple
 - (void)showHelp:sender
-{
-    if (!helpPanel)
+{   NSArray     *localizations;
+    NSString    *locale, *path, *helpFile;
+    int         l, i;
+
+    localizations = [NSBundle preferredLocalizationsFromArray:[[NSBundle mainBundle] localizations]];
+
+    for (l=0; l<[localizations count]; l++ )
     {
-        if (![NSBundle loadModelNamed:@"Help" owner:self])
-            NSLog(@"Cannot load Help interface file");	
+        locale = [[localizations objectAtIndex:l] stringByAppendingString:@".lproj"];
+        for (i=0; i<3; i++)
+        {
+            switch (i)
+            {
+                case 0: path = [[NSBundle mainBundle] resourcePath];
+                    helpFile = vhfPathWithPathComponents(path, locale, @"Cenon.pdf", nil);
+                    break;
+                case 1: path = vhfUserLibrary(APPNAME);
+                    helpFile = vhfPathWithPathComponents(path, @"Docu", locale, @"Cenon.pdf", nil);
+                    break;
+                case 2: path = vhfLocalLibrary(APPNAME);
+                    helpFile = vhfPathWithPathComponents(path, @"Docu", locale, @"Cenon.pdf", nil);
+                    break;
+                default:
+                    return;
+            }
+            if ( [[NSFileManager defaultManager] fileExistsAtPath:helpFile] )
+            {   [[NSWorkspace sharedWorkspace] openFile:helpFile];
+                l = [localizations count];
+                break;
+            }
+        }
     }
+
+    /*if ( !helpPanel && ![[NSBundle mainBundle] loadModelNamed:@"Help" owner:self] )
+            NSLog(@"Cannot load Help interface file");
     [helpPanel setFrameAutosaveName:@"HelpPanel"];
-    [helpPanel makeKeyAndOrderFront:sender];
+    [helpPanel makeKeyAndOrderFront:sender];*/
 }
 #endif
 
@@ -888,7 +960,7 @@ typedef enum
     [inspectorPanel setBecomesKeyOnlyIfNeeded:YES];
     [inspectorPanel orderFront:sender];
 }
-- inspectorPanel;
+- inspectorPanel
 {
     return inspectorPanel;
 }
@@ -904,7 +976,7 @@ typedef enum
     [transformPanel setFrameAutosaveName:@"TransformPanel"];
     [transformPanel makeKeyAndOrderFront:sender];
 }
-- transformPanel;
+- transformPanel
 {
     return transformPanel;
 }
@@ -915,12 +987,11 @@ typedef enum
     {
         if (![NSBundle loadModelNamed:@"ProjectSettingsPanel" owner:self])
             NSLog(@"Cannot load ProjectSettingsPanel interface file");	
-        [projectSettingsPanel init];
     }
     //[projectSettingsPanel setFrameAutosaveName:@"ProjectSettingsPanel"];
     [projectSettingsPanel makeKeyAndOrderFront:sender];
 }
-- projectSettingsPanel;
+- projectSettingsPanel
 {
     return projectSettingsPanel;
 }
