@@ -1,11 +1,12 @@
 /* Document.m
  * Cenon document class
  *
- * Copyright (C) 1996-2007 by vhf interservice GmbH
+ * Copyright (C) 1996-2008 by vhf interservice GmbH
  * Author: Georg Fleischmann
  *
  * Created:  1996-02-09
- * Modified: 2007-07-16
+ * Modified: 2008-10-21 (clean-up)
+ *           2008-07-24 (document units, update units)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -44,6 +45,9 @@
 #include "HPGLExportSub.h"
 #include "GerberExportSub.h"
 #include "DXFExportSub.h"
+#ifdef __APPLE__
+#    include "GSPropertyListSerialization.h"
+#endif
 
 #define XOFFSET		5.0 	// Offset of subsequent windows
 #define YOFFSET 	-20.0
@@ -175,6 +179,9 @@ static NSMutableArray *zoneList = nil;
 - (id)init
 {
     [super init];
+
+    baseUnit = -1;  // use unit from Preferences
+
     //[self registerForServicesMenu];
     return self;
 }
@@ -221,17 +228,17 @@ static NSMutableArray *zoneList = nil;
     return self;
 }
 
-/* modified: 2007-05-04
+/* modified: 2008-07-24 (load baseUnit, project settings)
  */
 + newFromFile:(NSString*)fileName
-{   Document		*newDocument = nil;
-    NSDictionary	*plist = nil;
-    NSString		*fileDirectory = nil, *outputFile = nil;
-    NSString		*directoryContainingDocument = nil;
-    NSFileManager	*fileManager = [NSFileManager defaultManager];
-    BOOL		isDirectory = NO;
-    DocView		*view;
-    NSRect		fRect;
+{   Document        *newDocument = nil;
+    NSDictionary    *plist = nil;
+    NSString        *fileDirectory = nil, *outputFile = nil;
+    NSString        *directoryContainingDocument = nil;
+    NSFileManager   *fileManager = [NSFileManager defaultManager];
+    BOOL            isDirectory = NO;
+    DocView         *view;
+    NSRect          fRect;
 
     if ([fileManager fileExistsAtPath:fileName isDirectory:&isDirectory])
     {
@@ -266,10 +273,17 @@ static NSMutableArray *zoneList = nil;
             [newDocument->printInfo setHorizontalPagination:NSAutoPagination];
             [newDocument->printInfo setVerticalPagination:NSAutoPagination];
 
-            newDocument->docVersion = [[plist objectForKey:@"DocVersion"] retain];
-            newDocument->docAuthor = [[plist objectForKey:@"Author"] retain];
-            newDocument->docCopyright = [[plist objectForKey:@"Copyright"] retain];
-            newDocument->docComment = [[plist objectForKey:@"Comment"] retain];
+            /* Project Info */
+            newDocument->docVersion   = [[plist objectForKey:@"DocVersion"] retain];
+            newDocument->docAuthor    = [[plist objectForKey:@"Author"]     retain];
+            newDocument->docCopyright = [[plist objectForKey:@"Copyright"]  retain];
+            newDocument->docComment   = [[plist objectForKey:@"Comment"]    retain];
+
+            /* Project Settings */
+            if ([plist objectForKey:@"DocSettings"])
+                newDocument->docSettingsDict = [[plist objectForKey:@"DocSettings"] mutableCopy];
+            if ([plist objectForKey:@"Unit"])
+                newDocument->baseUnit        = [plist intForKey:@"Unit"];
 
             directoryContainingDocument = fileDirectory ? fileDirectory
                                                         : [fileName stringByDeletingLastPathComponent];
@@ -317,7 +331,7 @@ static NSMutableArray *zoneList = nil;
             [view setDocument:newDocument];
             newDocument->magazineIndex = [plist intForKey:@"MagazineIndex"];
             [[NSNotificationCenter defaultCenter] postNotificationName:DocWindowDidChange
-                                                                object:newDocument];
+                                                                object:newDocument];    //FIXME: why without view set? It is missing at some places!
             [newDocument->scrollView setDocumentView:view];
             [view release];
             [newDocument->scrollView setDocument:newDocument];
@@ -417,15 +431,16 @@ static NSMutableArray *zoneList = nil;
 
 /*
  * Creates a new document from what is in the passed stream.
+ * modified: 2008-07-19 (-init allocated document)
  */
 + newFromList:(NSMutableArray*)list
-{   NSRect		frameRect, fRect;
-    DocView		*view;
-    TileScrollView	*sView;
-    DocWindow		*win;
-    Document		*newDocument;
+{   NSRect          frameRect, fRect;
+    DocView         *view;
+    TileScrollView  *sView;
+    DocWindow       *win;
+    Document        *newDocument;
 
-    newDocument = [super allocWithZone:[self newZone]];
+    newDocument = [[super allocWithZone:[self newZone]] init];
     if (![NSBundle loadModelNamed:@"Document" owner:newDocument])
     {	NSLog(@"Cannot load Document model");	
         [newDocument release];
@@ -707,28 +722,97 @@ static NSMutableArray *zoneList = nil;
     docVersion = [newVersion retain];
     [self setDirty:YES];
 }
-- (NSString*)docVersion		{ return docVersion; }
+- (NSString*)docVersion     { return docVersion; }
 - (void)setDocAuthor:(NSString*)newAuthor
 {
     [docAuthor release];
     docAuthor = [newAuthor retain];
     [self setDirty:YES];
 }
-- (NSString*)docAuthor		{ return docAuthor; }
+- (NSString*)docAuthor      { return docAuthor; }
 - (void)setDocCopyright:(NSString*)newCopyright
 {
     [docCopyright release];
     docCopyright = [newCopyright retain];
     [self setDirty:YES];
 }
-- (NSString*)docCopyright	{ return docCopyright; }
+- (NSString*)docCopyright   { return docCopyright; }
 - (void)setDocComment:(NSString*)newComment
 {
     [docComment release];
     docComment = [newComment copy];
     [self setDirty:YES];
 }
-- (NSString*)docComment		{ return docComment; }
+- (NSString*)docComment     { return docComment; }
+
+/* Document setting
+ * created:  2008-07-19
+ * modified: 2008-07-24
+ */
+- (NSMutableDictionary*)docSettingsDict
+{
+    if (!docSettingsDict)
+        docSettingsDict = [[NSMutableDictionary dictionary] retain];
+    return docSettingsDict;
+}
+- (void)setBaseUnit:(CenonUnit)unit { baseUnit = unit; }
+- (CenonUnit)baseUnitFlat           { return baseUnit; }
+- (CenonUnit)baseUnit
+{   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
+    return (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+}
+- (float)convertToUnit:(float)iValue
+{   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
+    CenonUnit       unit = (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+
+    switch ( unit )
+    {
+        case UNIT_MM:		return (iValue * 25.4/72.0);
+        case UNIT_INCH:		return (iValue / 72.0);
+        default:
+        case UNIT_POINT:	return iValue;
+    }
+    return iValue;
+}
+- (float)convertFrUnit:(float)uValue
+{   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
+    CenonUnit       unit = (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+
+    switch ( unit )
+    {
+        case UNIT_MM:		return (uValue / 25.4*72.0);
+        case UNIT_INCH:		return (uValue * 72.0);
+        default:
+        case UNIT_POINT:	return uValue;
+    }
+    return uValue;
+}
+- (float)convertMMToUnit:(float)mmValue
+{   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
+    CenonUnit       unit = (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+
+    switch ( unit )
+    {
+        default:
+        case UNIT_MM:		return mmValue;
+        case UNIT_INCH:		return mmValue / 25.4;
+        case UNIT_POINT:	return mmValue * 72.0/25.4;
+    }
+    return mmValue;
+}
+- (float)convertUnitToMM:(float)uValue
+{   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
+    CenonUnit       unit = (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+
+    switch ( unit )
+    {
+        default:
+        case UNIT_MM:		return uValue;
+        case UNIT_INCH:		return uValue * 25.4;
+        case UNIT_POINT:	return uValue / 72.0*25.4;
+    }
+    return uValue;
+}
 
 
 /*
@@ -934,18 +1018,18 @@ static NSMutableArray *zoneList = nil;
  * 3. move file name (old) file to backup (fileName.cenon -> fileName.cenon~)
  * 4. move temporary file to file name (#fileName.cenon -> fileName.cenon)
  *
- * modified: 2007-07-16
+ * modified: 2007-07-24 (save baseUnit and project settings)
  */
 - (BOOL)save
-{   BOOL		savedOk = NO;
-    NSString		*filename = [self filename];
-    NSString		*backupFilename = [filename stringByAppendingString:@"~"];
-    //NSString		*tmpFilename = [@"#" stringByAppendingString:filename];
-    NSString		*fileDirectory = nil;
-    NSFileManager	*fileManager = [NSFileManager defaultManager];
-    NSMutableDictionary	*fileContents;
-    BOOL		isDirectory = YES;
-    NSAutoreleasePool	*pool;
+{   BOOL                savedOk = NO;
+    NSString            *filename = [self filename];
+    NSString            *backupFilename = [filename stringByAppendingString:@"~"];
+    //NSString          *tmpFilename = [@"#" stringByAppendingString:filename];
+    NSString            *fileDirectory = nil;
+    NSFileManager       *fileManager = [NSFileManager defaultManager];
+    NSMutableDictionary *fileContents;
+    BOOL                isDirectory = YES;
+    NSAutoreleasePool   *pool;
 
     /* file not writable */
     if ( [fileManager fileExistsAtPath:filename] && ![fileManager isWritableFileAtPath:filename] )
@@ -981,53 +1065,73 @@ static NSMutableArray *zoneList = nil;
             //[fileContents setObject:propertyListFromInt([VGraphic currentGraphicIdentifier]) forKey:@"GraphicIdentifier"];
             [fileContents setObject:@"Cenon" forKey:@"Creator"];
             [fileContents setObject:[[NSCalendarDate date] descriptionWithCalendarFormat:@"%Y-%m-%d %H:%M"] forKey:@"CreationDate"];
+            /* Project Info */
             if (docVersion)
-                [fileContents setObject:docVersion forKey:@"DocVersion"];
+                [fileContents setObject:docVersion      forKey:@"DocVersion"];
             if (docAuthor)
-                [fileContents setObject:docAuthor forKey:@"Author"];
+                [fileContents setObject:docAuthor       forKey:@"Author"];
             if (docCopyright)
-                [fileContents setObject:docCopyright forKey:@"Copyright"];
+                [fileContents setObject:docCopyright    forKey:@"Copyright"];
             if (docComment)
-                [fileContents setObject:docComment forKey:@"Comment"];
+                [fileContents setObject:docComment      forKey:@"Comment"];
+            /* Project Settings */
+            if (baseUnit != -1)
+                [fileContents setObject:[NSString stringWithFormat:@"%d", baseUnit] forKey:@"Unit"];
+            if (docSettingsDict)
+                [fileContents setObject:docSettingsDict forKey:@"DocSettings"];
+
             if ( DOCUMENT_VERSION >= 101 )
                 [fileContents setObject:[[self documentView] propertyList] forKey:@"Data"];
             else
-            {
-                archiver = [[[NSArchiver alloc] initForWritingWithMutableData:[NSMutableData data]] autorelease];
+            {   archiver = [[[NSArchiver alloc] initForWritingWithMutableData:[NSMutableData data]] autorelease];
                 [[self documentView] encodeWithCoder:archiver];
                 [fileContents setObject:[archiver archiverData] forKey:@"Data"];
             }
 
 #ifdef __APPLE__
-            /* ASCII PropertyList... it just doesn't work, blame Apple! */
-            if ( Prefs_OSPropertyList &&
-                 [NSPropertyListSerialization propertyList:fileContents
-                                          isValidForFormat:NSPropertyListOpenStepFormat] )
-            {   NSData      *data;
-                NSString    *error = nil;
-                //NSDictionary *testplist = [NSDictionary dictionary];  // not even this works!
+            /* ASCII PropertyList... it just doesn't work on Mac OS X,
+             * so we use the class from the GNUstep project so long
+             */
+            if ( Prefs_OSPropertyList ) // ASCII PropertyList (most human readable + half the size of XML)
+            {
+                if ( [NSPropertyListSerialization propertyList:fileContents
+                                              isValidForFormat:NSPropertyListOpenStepFormat] )
+                {   NSData      *data;
+                    NSString    *error = nil;
+                    //NSDictionary *testplist = [NSDictionary dictionary];  // not even this works!
 
-                data = [NSPropertyListSerialization dataFromPropertyList:fileContents
-                                                                  format:NSPropertyListOpenStepFormat
-                                                        errorDescription:&error];
-                if (error)
-                {   NSLog(error);
-                    [error release];
+                    data = [NSPropertyListSerialization dataFromPropertyList:fileContents
+                                                                      format:NSPropertyListOpenStepFormat
+                                                            errorDescription:&error];
+                    if (error)
+                    {   NSLog(error); [error release]; }
+                    savedOk = [data writeToFile:filename atomically:YES];
+
+                    /* Apple stuff doesn't work:
+                    if ([NSPropertyListSerialization propertyList:testplist
+                                                isValidForFormat:NSPropertyListOpenStepFormat])
+                        NSLog(@"valid for ASCII");
+                    if ([NSPropertyListSerialization propertyList:testplist
+                                                 isValidForFormat:NSPropertyListXMLFormat_v1_0])
+                        NSLog(@"valid for XML");
+                    data = [NSPropertyListSerialization dataFromPropertyList:testplist
+                                                                      format:NSPropertyListOpenStepFormat
+                                                            errorDescription:&error];
+                    [data writeToFile:@"/Users/georg/Tempo/test.plist" atomically:YES];*/
                 }
-                savedOk = [data writeToFile:filename atomically:YES];
+                else
+                {   NSData      *data;
+                    NSString    *error = nil;
 
-                /*if ([NSPropertyListSerialization propertyList:testplist
-                                               isValidForFormat:NSPropertyListOpenStepFormat])
-                    NSLog(@"valid for ASCII");
-                if ([NSPropertyListSerialization propertyList:testplist
-                                               isValidForFormat:NSPropertyListXMLFormat_v1_0])
-                    NSLog(@"valid for XML");
-                data = [NSPropertyListSerialization dataFromPropertyList:testplist
-                                                                  format:NSPropertyListOpenStepFormat
-                                                        errorDescription:&error];
-                [data writeToFile:@"/Users/georg/Tempo/test.plist" atomically:YES];*/
+                    data = [GSPropertyListSerialization dataFromPropertyList:fileContents
+                                                                      format:NSPropertyListOpenStepFormat
+                                                            errorDescription:&error];
+                    if (error)
+                    {   NSLog(error); [error release]; }
+                    savedOk = [data writeToFile:filename atomically:YES];
+                }
             }
-            else	// default
+            else	// use native PropertyList format
 #endif
                 /* FIXME: converting dictionary to string needs a lot lot of memory !! */
                 savedOk = [fileContents writeToFile:filename atomically:YES];
@@ -1179,19 +1283,19 @@ static NSMutableArray *zoneList = nil;
 }
 
 - (void)printSeparation:(NSPrintOperation*)op
-{   NSColor		*sepColor;
-    NSPrintOperation	*sop=nil;
-    NSPrintInfo		*pi = [op printInfo], *spi;
-    NSMutableDictionary	*piDict = [pi dictionary];
-    NSAutoreleasePool	*pool0 = [NSAutoreleasePool new], *pool;
-    NSString		*cropPath;
-    NSString		*pStr=nil, *npStr=nil, *pExt=nil, *mStr, *paperName = [pi paperName];
-    NSMutableArray	*ml, *mlist=[NSMutableArray array], *layerList;
-    NSRect		mBounds, pageRect = NSZeroRect, newPageRect = NSZeroRect, oldFrame, visibleRect;
-    NSPoint		mPosition = NSZeroPoint, mOffset, lr, ll, ur, ul, center, dm, pOffset = {55.0, 55.0};
-    int			i, orientation = [pi orientation];
-    DocView		*docView = [self documentView];
-    float		width, height, scale = [docView scaleFactor];
+{   NSColor             *sepColor;
+    NSPrintOperation    *sop=nil;
+    NSPrintInfo         *pi = [op printInfo], *spi;
+    NSMutableDictionary *piDict = [pi dictionary];
+    NSAutoreleasePool   *pool0 = [NSAutoreleasePool new], *pool;
+    NSString            *cropPath;
+    NSString            *pStr=nil, *npStr=nil, *pExt=nil, *mStr, *paperName = [pi paperName];
+    NSMutableArray      *ml, *mlist=[NSMutableArray array], *layerList;
+    NSRect              mBounds, pageRect = NSZeroRect, newPageRect = NSZeroRect, oldFrame, visibleRect;
+    NSPoint             mPosition = NSZeroPoint, mOffset, lr, ll, ur, ul, center, dm, pOffset = {55.0, 55.0};
+    int                 i, orientation = [pi orientation];
+    DocView             *docView = [self documentView];
+    float               width, height, scale = [docView scaleFactor];
 
     cropPath = vhfPathWithPathComponents([[NSBundle mainBundle] resourcePath], CROPMARK_FOLDER, nil);
 
@@ -1593,12 +1697,12 @@ static NSMutableArray *zoneList = nil;
  * modified: 2005-09-01
  */
 - (void)printDocument:sender
-{   NSPrintOperation	*op;
-    NSPrintPanel	*pp;
-    NSPrintInfo		*pi;
-    DocView		*docView = [self documentView];
-    NSRect		visibleRect, oldFrame;
-    float		width, height, scale = [docView scaleFactor];
+{   NSPrintOperation    *op;
+    NSPrintPanel        *pp;
+    NSPrintInfo         *pi;
+    DocView             *docView = [self documentView];
+    NSRect              visibleRect, oldFrame;
+    float               width, height, scale = [docView scaleFactor];
 
     /* set scale to 1.0 */
     scale = [docView scaleFactor];
@@ -1622,7 +1726,7 @@ static NSMutableArray *zoneList = nil;
     /* single page document */
     else
     {
-        /* FIXME: GNUstep printing needs to be fixed for the code below */
+        /* FIXME: GNUstep printing needs to be fixed to work with the code below */
 #ifdef GNUSTEP_BASE_VERSION
         op = [NSPrintOperation printOperationWithView:[self documentView] printInfo:printInfo];
         [op setShowPanels:YES];
@@ -1769,6 +1873,7 @@ static NSMutableArray *zoneList = nil;
         [self save];*/
     [printInfo release];
     [fontObject release];
+    [docSettingsDict release];
     [super dealloc];
 }
 
