@@ -1,10 +1,11 @@
 /* VImage.m
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2008 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1998-03-22
- * modified: 2005-04-25 (clean up)
+ * modified: 2008-12-01 (-drawWithPrincipal: relief draw pale)
+ *           2008-07-20 (other file types added)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -49,17 +50,15 @@ static int currentGraphicIdentifier = 1;
 static BOOL checkImage(NSImage *anImage)
 {
     if ([anImage isValid])
-    { 
-	[anImage lockFocus];
-	[anImage unlockFocus];
-	return YES;
+    {
+        [anImage lockFocus];
+        [anImage unlockFocus];
+        return YES;
     }
     return NO;
 }
 
-#if 0	// not used yet, and not many of the methods available for GNUstep, yet
-//#if defined(GNUSTEP_BASE_VERSION) || defined(__APPLE__)
-
+#if defined(GNUSTEP_BASE_VERSION) || defined(__APPLE__) // GNUstep, Apple
 static NSBitmapImageFileType fileTypeFromVImageType(VImageFileType fileType)
 {
     switch (fileType)
@@ -102,10 +101,11 @@ static NSDictionary *propertiesForFileType(VImageFileType fileType, float compre
 }
 #endif
 
-static NSString *fileExtensionFromFileType(VImageFileType fileType)
++ (NSString*)fileExtensionForFileType:(VImageFileType)fileType
 {
     switch (fileType)
     {
+        default:
         case VImageTIFF_None:
         case VImageTIFF_LZW:
         case VImageTIFF_PackBits: return @"tiff";
@@ -113,7 +113,6 @@ static NSString *fileExtensionFromFileType(VImageFileType fileType)
         case VImageGIF:           return @"gif";
         case VImageJPEG:          return @"jpg";
         case VImagePNG:           return @"png";
-        //default:                  return @"tiff";
     }
     return nil;
 }
@@ -225,10 +224,10 @@ static VImageFileType fileTypeForTIFFCompression(NSTIFFCompression tiffCompressi
     [image setDataRetained:YES];
     if (checkImage(image))
     {
-	originalSize = [image size];
-	[image setScalesWhenResized:YES];
-	size = originalSize;
-	return self;
+        originalSize = [image size];
+        [image setScalesWhenResized:YES];
+        size = originalSize;
+        return self;
     }
     [self release];
     return nil;
@@ -269,13 +268,12 @@ static VImageFileType fileTypeForTIFFCompression(NSTIFFCompression tiffCompressi
 - (void)setParameter
 {
     selectedKnob = -1;
-    reliefType = RELIEF_LINEAR;
     clipPath = nil;
     thumbnail = NO;
     imageFile = [[NSString stringWithFormat:@"Image%d.tiff", identifier] retain];
     sourcePath = nil;
     fileType = VImageTIFF_LZW;
-    compressionFactor = 0.1;
+    compressionFactor = 0.9;
     compressionDirty = NO;
 }
 
@@ -286,22 +284,19 @@ static VImageFileType fileTypeForTIFFCompression(NSTIFFCompression tiffCompressi
     return originalSize.width / originalSize.height;
 }
 
-- (void)setSize:(NSSize)newSize		{ size = newSize; dirty = YES; }
-- (NSSize)size				{ return size; }
-- (NSSize)originalSize			{ return originalSize; }
+- (void)setSize:(NSSize)newSize         { size = newSize; dirty = YES; }
+- (NSSize)size                          { return size; }
+- (NSSize)originalSize                  { return originalSize; }
 
-- (void)setOrigin:(NSPoint)newOrigin	{ origin = newOrigin; dirty = YES; }
-- (NSPoint)origin			{ return origin; }
-
-- (void)setRelief:(BOOL)flag		{ relief = flag; dirty = YES; }
-- (BOOL)relief				{ return relief; }
-- (void)setReliefType:(int)type		{ reliefType = type; dirty = YES; }
-- (int)reliefType			{ return reliefType; }
+- (void)setOrigin:(NSPoint)newOrigin    { origin = newOrigin; dirty = YES; }
+- (NSPoint)origin                       { return origin; }
 
 - (NSString*)setName:(NSString*)str
-{
-    if (![[str pathExtension] isEqual:@"tiff"])
-        str = [NSString stringWithFormat:@"%@.tiff", str];
+{   NSString    *ext = [VImage fileExtensionForFileType:fileType];
+
+    if ( ! [[str pathExtension] isEqual:ext] )  // set correct path extension
+        str = [[str stringByDeletingPathExtension] stringByAppendingPathExtension:ext];
+
     [imageFile release];
     imageFile = [[NSString stringWithFormat:@"%@", str] retain];
     return imageFile;
@@ -458,14 +453,23 @@ static VImageFileType fileTypeForTIFFCompression(NSTIFFCompression tiffCompressi
         thumbImage = nil;
     }
 }
-- (BOOL)thumbnail			{ return thumbnail; }
+- (BOOL)thumbnail                           { return thumbnail; }
 
-- (void)setFileType:(VImageFileType)type	{ fileType = type; compressionDirty = YES; }
-- (VImageFileType)fileType;			{ return fileType; }
-- (void)setCompressionFactor:(float)f;	{ compressionFactor = f; compressionDirty = YES; }
-- (float)compressionFactor;		{ return compressionFactor; }
+- (void)setFileType:(VImageFileType)type
+{   NSString    *ext = [[self class] fileExtensionForFileType:type];
 
-- (id)clipPath				{ return clipPath; }
+    fileType = type;
+    compressionDirty = YES;
+    if ( ! [[imageFile pathExtension] isEqual:ext] )  // set correct path extension
+    {   NSString    *name = [[imageFile stringByDeletingPathExtension] stringByAppendingPathExtension:ext];
+        [self setName:name];
+    }
+}
+- (VImageFileType)fileType;                 { return fileType; }
+- (void)setCompressionFactor:(float)f;      { compressionFactor = f; compressionDirty = YES; }
+- (float)compressionFactor;                 { return compressionFactor; }
+
+- (id)clipPath                              { return clipPath; }
 
 /*
  * subclassed methods
@@ -512,6 +516,11 @@ static VImageFileType fileTypeForTIFFCompression(NSTIFFCompression tiffCompressi
  */
 - (void)splitTo:ulist
 {
+    if (!clipPath)
+    {   [ulist addObject:self];
+        return;
+    }
+    [ulist addObject:self]; // 2008-06-08: moved before clipPath in ulist
     [clipPath setSelected:YES];
     [clipPath setDirty:YES];
     [ulist addObject:clipPath];
@@ -521,7 +530,6 @@ static VImageFileType fileTypeForTIFFCompression(NSTIFFCompression tiffCompressi
 #ifdef USE_VHF_CLIPPING
     [clipImage release]; clipImage = nil;
 #endif
-    [ulist addObject:self];
 }
 
 /* antialiasing for OpenStep and GNUstep
@@ -1054,21 +1062,20 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
  * draws the image
  */
 - (void)drawWithPrincipal:principal
-{   NSPoint		p;
-    NSSize		currentSize;
-    NSRect		bounds;
-    float		scale = [principal scaleFactor];
-    NSBezierPath	*bPath;
-    NSRect		rect, visibleRect = [principal visibleRect];
-    BOOL		releaseImage = NO;
-    NSImage		*sepImage = nil; // color separated image
+{   NSPoint         p;
+    NSSize          currentSize;
+    NSRect          bounds, rect;
+    float           scale = [principal scaleFactor];
+    NSBezierPath    *bPath;
+    BOOL            releaseImage = NO;
+    NSImage         *sepImage = nil; // color separated image
 
     if (size.width < 1.0 || size.height < 1.0)
         return;
 
     bounds.origin = origin;
     bounds.size = size;
-    bounds.size.width *= scale;
+    bounds.size.width  *= scale;
     bounds.size.height *= scale;
 
     [super drawWithPrincipal:principal];	// set color
@@ -1096,7 +1103,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
                 [((thumbImage && VHFIsDrawingToScreen())?thumbImage:image) setSize:bounds.size];
             else
             {
-                p.x = origin.x + floor((bounds.size.width - currentSize.width) / 2.0 + 0.5);
+                p.x = origin.x + floor((bounds.size.width  - currentSize.width)  / 2.0 + 0.5);
                 p.y = origin.y + floor((bounds.size.height - currentSize.height) / 2.0 + 0.5);
             }
         }
@@ -1193,7 +1200,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
             }
             [bPath setWindingRule:NSEvenOddWindingRule];
             [bPath addClip];
-        }
+        }	// end clipPath
         /* ???
          if ([[image bestRepresentationForDevice:nil] isOpaque])
         {
@@ -1203,15 +1210,28 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
 
         /* display or print
          */
-#ifndef __APPLE__	// Apple has antialiasing
+#ifndef __APPLE__	// GNUstep/OpenStep (Apple has antialiasing)
         if (!thumbnail && VHFIsDrawingToScreen() && [self compositeAntialiased:scale toPoint:p])
+        {
+            if (clipPath)
+                PSgrestore();
             return;
+        }
 #endif
         if ([principal cache] && !sepImage) // not for colorseparation
         {   NSImage	*img = (thumbImage && VHFIsDrawingToScreen()) ? thumbImage : image;
 
             PSgsave();	// needed for GNUstep
-            [img compositeToPoint:p operation:NSCompositeSourceOver];
+                if ( [principal mustDrawPale] && relief )   //  Relief: draw a light green rect instead
+                {   NSColor	*col = [NSColor colorWithCalibratedRed:0.8 green:1.0 blue:0.8 alpha:1.0];
+
+                    [NSBezierPath setDefaultLineCapStyle: NSRoundLineCapStyle];
+                    [NSBezierPath setDefaultLineJoinStyle:NSRoundLineJoinStyle];
+                    [col set];
+                    [NSBezierPath fillRect:NSMakeRect(bounds.origin.x, bounds.origin.y, size.width, size.height)];
+                }   // FIXME: relief above should be removed and image should be composited pale
+                else
+                    [img compositeToPoint:p operation:NSCompositeSourceOver];
             PSgrestore();
         }
         /* Color separation
@@ -1223,7 +1243,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
 #ifdef __APPLE__	// Apple works
             rect = bounds;
 #else			// workaround for OpenStep
-            rect = NSIntersectionRect(bounds, visibleRect);
+            rect = NSIntersectionRect(bounds, [principal visibleRect]);
 #endif
             if (rect.size.width && rect.size.height)
             {   NSPoint	pOld = p;
@@ -1238,6 +1258,13 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
                 {   [sepImage compositeToPoint:p fromRect:rect operation:NSCompositeSourceOver];
                     [sepImage release];
                 }
+                else if ( [principal mustDrawPale] && relief )  // draw a light green rect instead
+				{   NSColor	*col = [NSColor colorWithCalibratedRed:0.8 green:1.0 blue:0.8 alpha:1.0];
+
+					[col set];
+					//[NSBezierPath fillRect:NSMakeRect(rect.origin.x, rect.origin.y, rect.size.width, rect.size.height)];
+					[NSBezierPath fillRect:NSMakeRect(bounds.origin.x, bounds.origin.y, bounds.size.width, bounds.size.height)];
+				}   // FIXME: relief above should be removed and image should be composited pale
                 else
                     [((thumbImage && VHFIsDrawingToScreen()) ? thumbImage : image)
                      compositeToPoint:p fromRect:rect operation:NSCompositeSourceOver];
@@ -1253,7 +1280,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
 #ifdef __APPLE__	// Apple works
             rect = bounds;
 #else			// workaround for OpenStep
-            rect = NSIntersectionRect(bounds, visibleRect);
+            rect = NSIntersectionRect(bounds, [principal visibleRect]);
 #endif
             if (rect.size.width && rect.size.height)
             {   rect.size.width *= scale; // important
@@ -1539,7 +1566,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
         knobRect.origin.y = p.y - controlsize/2.0;
         if (!NSIsEmptyRect(NSIntersectionRect(hitRect , knobRect)))
         {   *pt = p;
-            selectedKnob = i;
+            //selectedKnob = i;
             return YES;
         }
     }
@@ -1570,13 +1597,41 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
     return NO;
 }
 
+/*
+ * modified: 2008-06-27
+ */
 - (BOOL)hit:(NSPoint)p fuzz:(float)fuzz
-{   NSRect	aRect, bRect;
-    int		i;
+{   NSRect  aRect, bRect;
+    int     i;
+
+    if (clipPath)   // clipped image -> we use the clip path for selection
+    {   BOOL    flag;
+
+        if ([clipPath respondsToSelector:@selector(setFilled:optimize:)])
+            [clipPath setFilled:YES optimize:NO];
+        else
+            [clipPath setFilled:YES];
+        flag = [clipPath hit:p fuzz:fuzz];
+        if ([clipPath respondsToSelector:@selector(setFilled:optimize:)])
+            [clipPath setFilled:NO optimize:NO];
+        else
+            [clipPath setFilled:NO];
+        return flag;
+    }
+    if (!Prefs_SelectByBorder)
+    {
+        bRect.origin.x    = origin.x - fuzz;
+        bRect.origin.y    = origin.y - fuzz;
+        bRect.size.width  = size.width  + 2.0*fuzz;
+        bRect.size.height = size.height + 2.0*fuzz;
+        if ( NSPointInRect(p, bRect) )
+            return YES;
+        return NO;
+    }
 
     aRect.origin.x = floor(p.x - fuzz);
     aRect.origin.y = floor(p.y - fuzz);
-    aRect.size.width = ceil(p.x + fuzz) - aRect.origin.x;
+    aRect.size.width  = ceil(p.x + fuzz) - aRect.origin.x;
     aRect.size.height = ceil(p.y + fuzz) - aRect.origin.y;
     for (i=0; i<PTS_RECTANGLE; i++)
     {	NSPoint	p0, p1;
@@ -1586,9 +1641,9 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
 
         bRect.origin.x = floor(Min(p0.x, p1.x) - 1);
         bRect.origin.y = floor(Min(p0.y, p1.y) - 1);
-        bRect.size.width = ceil(Max(p0.x, p1.x) + 2) - bRect.origin.x;
+        bRect.size.width  = ceil(Max(p0.x, p1.x) + 2) - bRect.origin.x;
         bRect.size.height = ceil(Max(p0.y, p1.y) + 2) - bRect.origin.y;
-        if ( !NSIsEmptyRect(NSIntersectionRect(aRect , bRect)) )
+        if ( !NSIsEmptyRect(NSIntersectionRect(aRect, bRect)) )
             return YES;
     }
     return NO;
@@ -1649,12 +1704,14 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
         unsigned char		*data[5], *cmykplanes[5];
         NSImage			*cmykImage;
         NSBitmapImageRep	*cmykbitmap;
-        NSString		*cmykFilename = nil;
+        NSString		*cmykFilename = nil, *string;
+        NSAutoreleasePool	*pool = [NSAutoreleasePool new];
 
         if (![VImage isAcceptedFileType:fileType])
             fileType = VImageTIFF_None;	// default
 
-        cmykFilename = [NSString stringWithFormat:@"%@/%@_cmyk.tiff", [directory stringByDeletingLastPathComponent], [imageFile stringByDeletingPathExtension]];
+        string = [NSString stringWithFormat:@"%@_cmyk.tiff", [imageFile stringByDeletingPathExtension]];
+        cmykFilename = vhfPathWithPathComponents([directory stringByDeletingLastPathComponent], string, nil);
 
         if (![fileManager isWritableFileAtPath:[directory stringByDeletingLastPathComponent]])
         {
@@ -1663,6 +1720,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
             {   [image release];
                 image = nil;
             }
+            [pool release];
             return NO;
         }
         /* check if allways cmyk */
@@ -1676,6 +1734,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
             imageData = [image TIFFRepresentationUsingCompression:tiffCompressionFromVImageType(fileType)
                                                            factor:0.0];
             [imageData writeToFile:cmykFilename atomically:NO];
+            [pool release];
             return YES;
         }
         if ([bmImageRep colorSpaceName] != NSCalibratedRGBColorSpace)
@@ -1684,6 +1743,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
             {   [image release];
                 image = nil;
             }
+            [pool release];
             return NO;
         }
 
@@ -1707,6 +1767,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
             }
             if (bps != 8)
                 NSLog(@"VImage, saveToCMYKTiff: unsupported bits per sample %d", bps);
+            [pool release];
             return NO;
         }
 
@@ -1746,36 +1807,64 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
                 if (spp <= 2)
                     *((cmykplanes[3]+by+x)) = 255 - ((pl) ? *((data[0]+lby+x)) : *((data[0]+lby+x*spp))); // w k
                 else
-                {   long	c, m, y, k = 0;
+                {   //long	c, m, yc, k = 0;
+                    long	r, g, b, ci, mi, yi, ki;
+                    float	rf, gf, bf;
+                    NSColor	*cmykCol, *rgbCol;
+//#if 0
+                    /* use OpenStep color conversion */
+                    r = ((pl) ? *((data[0]+lby+x)) : *((data[0]+lby+x*spp)));
+                    g = ((pl) ? *((data[1]+lby+x)) : *((data[0]+lby+x*spp+1)));
+                    b = ((pl) ? *((data[2]+lby+x)) : *((data[0]+lby+x*spp+2)));
 
+                    rf = (float)r/255.0;
+                    gf = (float)g/255.0;
+                    bf = (float)b/255.0;
+                    rgbCol = [NSColor colorWithCalibratedRed:rf green:gf blue:bf alpha:1.0];
+                    cmykCol = [rgbCol colorUsingColorSpaceName:NSDeviceCMYKColorSpace];
+                    ci = ((long)([cmykCol cyanComponent] * 255.0));
+                    mi = ((long)([cmykCol magentaComponent] * 255.0));
+                    yi = ((long)([cmykCol yellowComponent] * 255.0));
+                    ki = ((long)([cmykCol blackComponent] * 255.0));
+                    *((cmykplanes[0]+by+x)) = ci;
+                    *((cmykplanes[1]+by+x)) = mi;
+                    *((cmykplanes[2]+by+x)) = yi;
+                    *((cmykplanes[3]+by+x)) = ki;
+//#endif
+#if 0
                     c = 255 - ((pl) ? *((data[0]+lby+x)) : *((data[0]+lby+x*spp))); // 255-r = c
                     m = 255 - ((pl) ? *((data[1]+lby+x)) : *((data[0]+lby+x*spp+1))); // 255-g = m
-                    y = 255 - ((pl) ? *((data[2]+lby+x)) : *((data[0]+lby+x*spp+2))); // 255-b = y
+                    yc = 255 - ((pl) ? *((data[2]+lby+x)) : *((data[0]+lby+x*spp+2))); // 255-b = y
 
-                    if (c >= 253 && m >= 253 && y >= 253)
+                    if (c >= 253 && m >= 253 && yc >= 253)
                     {   /* realy black */
                         c = 127;
                         m = y = 0;
                         k = 255;
                     }
-                    else if (Min(c, Min(m, y)) > 1)
-                    {   long	min = Min(c, Min(m, y));
 
-                        //if (c+m+y < 510) // < 200 % // nicht gut
-                        //    min = ((int)(2.0*(float)min/3.0));
-                        k = min;
+                    else if (Min(c, Min(m, yc)) > 1)
+                    {   long	min = Min(c, Min(m, yc));
+
+                        k = min; // /2.0; // zu dunkel bei den mittleren Farben
+// all for MetalSurface conversion
+//k -= 5;
+//if (k < 0) k = 0;
+
                         c -= min;
+//c += 5;
+//if (c > 255) c = 255;
                         m -= min;
-                        y -= min;
+//m *= 0.8;
+                        yc -= min;
+//yc *= 0.8;
                    }
-/*k = Max(0, k-20);
-c = Max(0, c-20);
-m = Max(0, m-20);
-y = Max(0, y-20);*/
+
                     *((cmykplanes[0]+by+x)) = c;
                     *((cmykplanes[1]+by+x)) = m;
-                    *((cmykplanes[2]+by+x)) = y;
+                    *((cmykplanes[2]+by+x)) = yc;
                     *((cmykplanes[3]+by+x)) = k;
+#endif
                 }
             }
         }
@@ -1792,6 +1881,7 @@ y = Max(0, y-20);*/
         imageData = [cmykImage TIFFRepresentationUsingCompression:tiffCompressionFromVImageType(fileType)
                                                            factor:0.0];
         [imageData writeToFile:cmykFilename atomically:NO];
+        [pool release];
         return YES;
     }
     if (thumbnail && thumbImage)
@@ -1810,12 +1900,13 @@ y = Max(0, y-20);*/
     NSString		*thFilename = [directory stringByAppendingPathComponent:[imageFile lastPathComponent]];
     NSFileManager	*fileManager = [NSFileManager defaultManager];
 
+//[self saveToCMYKTiff:directory]; /// hack debug test only !
+
     if (![VImage isAcceptedFileType:fileType])
         fileType = VImageTIFF_None;	// default
 
     if (thumbnail && thumbImage)
-    {   NSString	*str = [NSString stringWithFormat:@"%@_.%@", [imageFile stringByDeletingPathExtension],
-                                                                     [imageFile pathExtension]];
+    {   NSString	*str = [NSString stringWithFormat:@"%@_.tiff", [imageFile stringByDeletingPathExtension]];
         thFilename = [directory stringByAppendingPathComponent:[str lastPathComponent]];
         if (![fileManager fileExistsAtPath:filename])
             [[thumbImage TIFFRepresentationUsingCompression:NSTIFFCompressionNone factor:0.0]
@@ -1870,14 +1961,25 @@ y = Max(0, y-20);*/
     else if (image && ![fileManager fileExistsAtPath:filename])
     {   NSData	*data;
 
-//#if !defined(GNUSTEP_BASE_VERSION) && !defined(__APPLE__)	// OpenStep 4.2
+#       if !defined(GNUSTEP_BASE_VERSION) && !defined(__APPLE__)    // OpenStep
         data = [image TIFFRepresentationUsingCompression:tiffCompressionFromVImageType(fileType)
                                                   factor:0.0];
-/*#else
-        data = [NSBitmapImageRep representationOfImageRepsInArray:[image representations]
-                                                        usingType:fileTypeFromVImageType(fileType)
-                                 properties:propertiesForFileType(fileType, compressionFactor)];
-#endif*/
+#       else    // GNUstep, Apple
+        if ( (fileType >= VImageTIFF_None && fileType <= VImageTIFF_PackBits) ) // TIFF
+        {
+            data = [image TIFFRepresentationUsingCompression:tiffCompressionFromVImageType(fileType)
+                                                      factor:0.0];
+#           ifdef __APPLE__
+              if (!data)  // for example 4-bit TIFF
+                  data = [image TIFFRepresentationUsingCompression:NSTIFFCompressionNone factor:0.0];
+#           endif
+        }
+        else    // GIF, JPG, PNG, ...
+        {   data = [NSBitmapImageRep representationOfImageRepsInArray:[image representations]
+                                     usingType:fileTypeFromVImageType(fileType)
+                                     properties:propertiesForFileType(fileType, compressionFactor)];
+        }
+#       endif
         [data writeToFile:filename atomically:NO];
         if (sourcePath)
             [sourcePath release];
@@ -1900,7 +2002,6 @@ y = Max(0, y-20);*/
     [aCoder encodeValuesOfObjCTypes:"@", &imageFile];
     [aCoder encodeValuesOfObjCTypes:"@", &image];
     [aCoder encodeValuesOfObjCTypes:"@", &thumbImage];
-    [aCoder encodeValuesOfObjCTypes:"ci", &relief, &reliefType];
     [aCoder encodeValuesOfObjCTypes:"@", &clipPath];
     [aCoder encodeValuesOfObjCTypes:"cif", &thumbnail, &fileType , &compressionFactor];
     [aCoder encodeValuesOfObjCTypes:"@", &sourcePath];
@@ -1926,8 +2027,9 @@ y = Max(0, y-20);*/
         identifier = currentGraphicIdentifier++; // the next one ! - need a new name !
     [aDecoder decodeValuesOfObjCTypes:"@", &imageFile];
     if (![imageFile hasSuffix:@".tiff"])
-    {   image = [[NSUnarchiver unarchiveObjectWithFile:[[[[[self class] currentView] document] filename] stringByAppendingPathComponent:imageFile]] retain];
-        [imageFile release]; // war autorelease
+    {   NSString    *path = [[[[[self class] currentView] document] filename] stringByAppendingPathComponent:imageFile];
+        image = [[NSUnarchiver unarchiveObjectWithFile:path] retain];
+        [imageFile release];
         imageFile = [[imageFile stringByAppendingString:@".tiff"] retain];
     }
     if (version >= 3)
@@ -1938,11 +2040,11 @@ y = Max(0, y-20);*/
     {   image = [[NSImage allocWithZone:(NSZone *)[self zone]] init];
         [image addRepresentations:[NSImageRep imageRepsWithContentsOfFile:[[[[[self class] currentView] document] filename] stringByAppendingPathComponent:imageFile]]];
         [image setDataRetained:YES];
-	[image setScalesWhenResized:YES];
+        [image setScalesWhenResized:YES];
     }
     imageFile = [[NSString stringWithFormat:@"Image%d.tiff", identifier] retain]; // rename -> new VImage
 
-    if (version >= 2)	// 2000-11-07
+    if (version >= 2 && version < 4)	// 2000-11-07 - 2008-03-18
         [aDecoder decodeValuesOfObjCTypes:"ci", &relief, &reliefType];
 
     if (version >= 3)
@@ -1954,6 +2056,7 @@ y = Max(0, y-20);*/
 }
 
 /* archiving with property list
+ * modified: 2008-07-19 (in init  != tiff case removed, because it was from archiver an just preventing success)
  */
 - (id)propertyList
 {   NSMutableDictionary	*plist = [super propertyList];
@@ -1965,8 +2068,6 @@ y = Max(0, y-20);*/
         imageFile = [[NSString stringWithFormat:@"Image%d.tiff", identifier] retain];
     [plist setObject:imageFile forKey:@"imageFile"];
     [plist setInt:identifier forKey:@"identifier"];
-    if (relief) [plist setObject:@"YES" forKey:@"relief"];
-    [plist setInt:reliefType forKey:@"reliefType"];
     if (clipPath)
         [plist setObject:[clipPath propertyList] forKey:@"clipPath"];
     if (thumbnail) [plist setObject:@"YES" forKey:@"thumbnail"];
@@ -1976,8 +2077,8 @@ y = Max(0, y-20);*/
     return plist;
 }
 - (id)initFromPropertyList:(id)plist inDirectory:(NSString *)directory
-{   id		plistObject, obj;
-    NSString	*className, *str, *path;
+{   id          plistObject, obj;
+    NSString    *className, *str = nil, *path;
 
     [self setParameter];
     [super initFromPropertyList:plist inDirectory:directory];
@@ -1992,37 +2093,24 @@ y = Max(0, y-20);*/
     /* thumbnail */
     thumbnail = ([plist objectForKey:@"thumbnail"] ? YES : NO);
     if (thumbnail)
-        str = [NSString stringWithFormat:@"%@_.%@", [imageFile stringByDeletingPathExtension],
-                                                    [imageFile pathExtension]];
+        str = [NSString stringWithFormat:@"%@_.tiff", [imageFile stringByDeletingPathExtension]];
     if (thumbnail &&
         [[NSFileManager defaultManager] fileExistsAtPath:[directory stringByAppendingPathComponent:str]])
     {   thumbImage = [[NSImage allocWithZone:(NSZone *)[self zone]] init];
         path = [directory stringByAppendingPathComponent:str];
         [thumbImage addRepresentations:[NSImageRep imageRepsWithContentsOfFile:path]];
         [thumbImage setDataRetained:YES];
-	[thumbImage setScalesWhenResized:YES];
+        [thumbImage setScalesWhenResized:YES];
     }
-    // FIXME: we should only convert to TIFF, if other formats are not available !
-    else if (![imageFile hasSuffix:@".tiff"])	// convert to tiff
-    {
-        path = [directory stringByAppendingPathComponent:imageFile];
-        image = [[NSUnarchiver unarchiveObjectWithFile:path] retain];
-        [imageFile release];	// war autorelease
-        imageFile = [[imageFile stringByAppendingString:@".tiff"] retain];
-    }
-    else
     {   image = [[NSImage allocWithZone:(NSZone *)[self zone]] init];
         path = [directory stringByAppendingPathComponent:imageFile];
         [image addRepresentations:[NSImageRep imageRepsWithContentsOfFile:path]];
         [image setDataRetained:YES];
-	[image setScalesWhenResized:YES];
+        [image setScalesWhenResized:YES];
     }
     sourcePath = [[directory stringByAppendingPathComponent:imageFile] retain];
     originalSize = [((thumbImage) ? thumbImage : image) size];
 
-    relief = ([plist objectForKey:@"relief"] ? YES : NO);
-    reliefType = [plist intForKey:@"reliefType"];
-
     /* clipPath */
     plistObject = [plist objectForKey:@"clipPath"];
     if (plistObject)
@@ -2041,7 +2129,10 @@ y = Max(0, y-20);*/
         fileType = [plist intForKey:@"fileType"];
     if (!fileType)
         fileType = VImageTIFF_None;	// default
-    compressionFactor = [plist floatForKey:@"compressionFactor"];
+    if ([plist objectForKey:@"compressionFactor"])
+        compressionFactor = [plist floatForKey:@"compressionFactor"];
+    else
+        compressionFactor = 0.9;    // 10% compression
 
     return self;
 }
