@@ -6,7 +6,7 @@
 # - call 'Convertgmodels1.sh' to convert special characters
 #
 # Author:   Georg Fleischmann
-# modified: 2006-11-15
+# modified: 2008-12-09
 #
 
 CENON=`echo $0 | sed -e 's|Nib2gmodel.sh||'`
@@ -31,7 +31,7 @@ do
     done
 
     # InspectorPanel
-    for nib in InspectorPanel IPAllFilling IPAllLayers IPAllStrokeWidth IPAllText \
+    for nib in InspectorPanel IPAllFilling IPAllLayers IPAllStrokeWidth IPAllAcc IPAccText \
 	       IPArc IPCrosshairs IPCurve IPGroup IPImage IPLine IPLine3D IPPolyLine IPMark \
 	       IPPath IPRectangle IPThread IPText IPTextPath IPWeb IPSinking
     do
