@@ -1,11 +1,12 @@
 /* vhf2DFunctions.m
  * vhf 2-D graphic functions
  *
- * Copyright (C) 1996-2004 by vhf interservice GmbH
+ * Copyright (C) 1996-2008 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-19
- * modified: 2004-08-04
+ * modified: 2008-02-17 (some clean-up)
+ *           2008-08-18 (vhfIsIntValueInArray() added, valueInArray() is deprecated now)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -57,6 +58,51 @@ float sqrDistancePointLine(const NSPoint *p0, const NSPoint *p1, const NSPoint *
 
     return pointOnLineClosestToPoint(*p0, *p1, *point, &iPoint);
 }
+float vhfSqrDistancePointLine(NSPoint p0, NSPoint p1, NSPoint point)
+{   NSPoint	iPoint;
+
+    return pointOnLineClosestToPoint(p0, p1, point, &iPoint);
+}
+
+/* modified: 09.11.96
+ */
+float distancePointArc(NSPoint pt, NSPoint cp, float r, float ba, float angle)
+{   float	dist;		// the distance to the full circle
+    float	dx, dy, c;
+    float	an;		// angle of point
+    NSPoint	p0, pa;
+    float	da, db;		// distances to endpoints
+    float	ea = ba + angle;
+
+    /* we need positive angles with ea > ba */
+    if (angle < 0.0)
+    {   float	v; v=ba; ba = ea; ea = v;}
+    if (ba < 0.0) ba += 360.0;
+    if (ba >= 360.0) ba -= 360.0;
+    ea = ba + Abs(angle);
+
+    dx = pt.x - cp.x; dy = pt.y - cp.y;
+    c = sqrt(dx*dx+dy*dy);
+    dist = Diff(r, c);	/* distance to full circle */
+
+    if (Diff(ba, ea) >= 360.0)	/* this is a full arc */
+        return dist;
+
+    /* compare angle of p */
+    an = vhfAngleOfPointRelativeCenter(pt, cp);
+    if (an < ba) an += 360.0;
+    if (an >= ba && an <= ea)	/* we are inside the angles */
+        return dist;
+
+    /* distance to endpoints */
+    p0.x = cp.x + r; p0.y = cp.y;
+    pa = vhfPointAngleFromRefPoint(cp, p0, ba);
+    da = SqrDistPoints(pa, pt);
+    pa = vhfPointAngleFromRefPoint(cp, p0, ea);
+    db = SqrDistPoints(pa, pt);
+
+    return sqrt((da < db) ? da : db);	/* distance to endpoint */
+}
 
 /*
  * created:  1996-03-29
@@ -437,6 +483,11 @@ BOOL vhfContainsRect(NSRect rect1, NSRect rect2)
     return NSContainsRect(rect1, rect2);
 }
 
+
+/*
+ * Rotation
+ */
+
 /* cw
  */
 NSPoint vhfPointRotatedAroundCenter(NSPoint p, float a, NSPoint cp)
@@ -464,6 +515,78 @@ void vhfRotatePointAroundCenter(NSPoint *p, NSPoint cp, float a)
     p->y = np.y + cp.y;
 }
 
+
+/*
+ * Angle
+ */
+
+/* calculate the angle of p relative cp
+ */
+float vhfAngleOfPointRelativeCenter(NSPoint p, NSPoint cp)
+{   float a, dx = p.x - cp.x, dy = p.y - cp.y;
+
+    if (!dx)
+        a = (dy >= 0) ? 90.0 : 270.0;
+    else
+    {	a = RadToDeg(atan(dy / dx));
+        if (dx<0) a += 180; /* 2, 3 */
+        if (dx>0 && dy<0) a += 360;	/* 4 */
+    }
+    return a;
+}
+
+/* calculates a point on the radius with a distance of angle (degree) from a reference point
+ */
+NSPoint vhfPointAngleFromRefPoint(NSPoint cp, NSPoint refP, float angle)
+{
+    return vhfPointRotatedAroundCenter(refP, angle, cp);
+}
+
+/* 1996-06-??
+ * return the angle between the 3 points ccw
+ */
+float vhfAngleBetweenPoints(const NSPoint start, const NSPoint middle, const NSPoint end)
+{   double	dx, dy;
+    float	angle0, angle1;
+
+    /* calculate angle of start to 0 degree
+     */
+    dx = start.x - middle.x;
+    dy = start.y - middle.y;
+    if (!dx)
+        angle0 = (dy >= 0) ? 90.0 : 270;
+    else
+    {	angle0 = RadToDeg(atan(dy / dx));
+        if (dx<0)		/* 2nd, 3rd quadrant */
+            angle0 += 180;
+        if (dx>0 && dy<0)	/* 4th quadrant */
+            angle0 += 360;
+    }
+
+    /* calculate angle of end to 0 degree
+     */
+    dx = end.x - middle.x;
+    dy = end.y - middle.y;
+    if (!dx)
+        angle1 = (dy >= 0) ? 90.0 : 270;
+    else
+    {	angle1 = RadToDeg(atan(dy / dx));
+        if (dx<0)		/* 2nd, 3rd quadrant */
+            angle1 += 180;
+        if (dx>0 && dy<0)	/* 4th quadrant */
+            angle1 += 360;
+    }
+
+    if (angle1 < angle0)
+        angle1 += 360;
+    return angle1 - angle0;
+}
+
+
+/*
+ * Sorting
+ */
+
 /* created:   19.03.96
  * modified:  
  * purpose:   sort array of points with increasing distance to p0
@@ -475,20 +598,20 @@ void sortPointArray(NSPoint *pArray, int cnt, NSPoint p0)
 {   int		i;
 
     for (i=0; i<cnt-1; i++)
-    {	int	j, jMin;
-        float	lastDist, newDist;
-        NSPoint	p;
+    {	int     j, jMin;
+        float   lastDist, newDist;
+        NSPoint p;
 
         jMin = cnt;
-        lastDist=SqrDistPoints(pArray[i], p0);
+        lastDist = SqrDistPoints(pArray[i], p0);
         for (j=i+1; j<cnt; j++)
         {
-            if ((newDist=SqrDistPoints(pArray[j], p0)) < lastDist)
+            if ((newDist = SqrDistPoints(pArray[j], p0)) < lastDist)
             {	lastDist = newDist;
                 jMin = j;
             }
         }
-        if (jMin<cnt)
+        if (jMin < cnt)
         {   p = pArray[i];
             pArray[i] = pArray[jMin];
             pArray[jMin] = p;
@@ -572,11 +695,11 @@ BOOL pointWithToleranceInArray(NSPoint point, float tol, const NSPoint *array, i
     return NO;
 }
 
-/* created:  19.06.96
- * modified:
+/* created:  1996-06-19
+ * modified: 
  * purpose:  return YES, if value is in array
  */
-BOOL valueInArray(int val, int* array, int cnt)
+BOOL valueInArray(int val, int* array, int cnt) // DEPRECATED, use vhfIsIntValueInArray()
 {   int	i;
 
     for (i=0; i<cnt; i++)
@@ -584,11 +707,21 @@ BOOL valueInArray(int val, int* array, int cnt)
             return YES;
     return NO;
 }
+/* created:  1996-06-19
+ * modified: 2008-08-18
+ * purpose:  return YES, if value is in array
+ */
+BOOL vhfIsIntValueInArray(int val, int* array, int cnt)
+{   int	i;
+    
+    for (i=0; i<cnt; i++)
+        if ( array[i] == val )
+            return YES;
+    return NO;
+}
 
 /* created:   1993-07-08
  * modified:  2001-02-15
- * purpose:   
- * parameter: 
  */
 int removePointFromArray(NSPoint p, NSPoint *pArray, int cnt)
 {   int	i, j;
@@ -630,67 +763,10 @@ int vhfFilterPoints(NSPoint *pArray, int cnt, float tol)
     return cnt;
 }
 
-/* calculate the angle of p relative cp
+
+/*
+ * Bezier
  */
-float vhfAngleOfPointRelativeCenter(NSPoint p, NSPoint cp)
-{   float a, dx = p.x - cp.x, dy = p.y - cp.y;
-
-    if (!dx)
-        a = (dy >= 0) ? 90.0 : 270.0;
-    else
-    {	a = RadToDeg(atan(dy / dx));
-        if (dx<0) a += 180; /* 2, 3 */
-        if (dx>0 && dy<0) a += 360;	/* 4 */
-    }
-    return a;
-}
-
-/* calculates a point on the radius with a distance of angle (degree) from a reference point
- */
-NSPoint vhfPointAngleFromRefPoint(NSPoint cp, NSPoint refP, float angle)
-{
-    return vhfPointRotatedAroundCenter(refP, angle, cp);
-}
-
-/* 1996-06-??
- * return the angle between the 3 points ccw
- */
-float vhfAngleBetweenPoints(const NSPoint start, const NSPoint middle, const NSPoint end)
-{   double	dx, dy;
-    float	angle0, angle1;
-
-    /* calculate angle of start to 0 degree
-     */
-    dx = start.x - middle.x;
-    dy = start.y - middle.y;
-    if (!dx)
-        angle0 = (dy >= 0) ? 90.0 : 270;
-    else
-    {	angle0 = RadToDeg(atan(dy / dx));
-        if (dx<0)		/* 2nd, 3rd quadrant */
-            angle0 += 180;
-        if (dx>0 && dy<0)	/* 4th quadrant */
-            angle0 += 360;
-    }
-
-    /* calculate angle of end to 0 degree
-     */
-    dx = end.x - middle.x;
-    dy = end.y - middle.y;
-    if (!dx)
-        angle1 = (dy >= 0) ? 90.0 : 270;
-    else
-    {	angle1 = RadToDeg(atan(dy / dx));
-        if (dx<0)		/* 2nd, 3rd quadrant */
-            angle1 += 180;
-        if (dx>0 && dy<0)	/* 4th quadrant */
-            angle1 += 360;
-    }
-
-    if (angle1 < angle0)
-        angle1 += 360;
-    return angle1 - angle0;
-}
 
 /* created:   
  * modified:  
@@ -793,6 +869,127 @@ BOOL vhfGetGradientOfCurveAt(NSPoint *p, NSPoint pc[4], float t)
     return YES;
 }
 
+/* flatten curve
+ *
+ * return: number of points in pts
+ *         pts has to be freed using NSZoneFree(NSDefaultMallocZone(), pts)
+ *
+ * modified: 2007-09-21
+ */
+int vhfConvertCurveToLines(NSPoint pc[4], float flatness, NSPoint **pts)
+{   NSPoint     *apPointList[3], *pPointRead, *pPointWrite, curvePoint;
+    double      aLength[3], ax, ay, bx, by, cx, cy, t, dist, maxdist;
+    int         i, i2, n = 1;
+    NSZone      *zone = NSDefaultMallocZone();
+
+    if ( flatness < 0.02 )
+        flatness = 0.02;
+    (apPointList[0]) = NSZoneMalloc(zone, (ldexp(3, 9)+1) * sizeof(NSPoint));
+    (apPointList[1]) = NSZoneMalloc(zone, (ldexp(3, 9)+1) * sizeof(NSPoint));
+    apPointList[2]   = 0;
+
+    * apPointList[0]    = pc[0];
+    *(apPointList[0]+1) = pc[1];
+    *(apPointList[0]+2) = pc[2];
+    *(apPointList[0]+3) = pc[3];
+
+    /* split curve in two curves until the wanted resolution has been achieved */
+    do
+    {   maxdist = dist = 0.0;
+
+        if ( n >= 9 && !(apPointList[1] = NSZoneRealloc(zone, apPointList[1], (ldexp(3,n)+1) * sizeof(NSPoint))) )
+        {   NSZoneFree(zone, apPointList[0]);
+            return 0;
+        }
+        pPointRead  = apPointList[0];
+        pPointWrite = apPointList[1];
+        *pPointWrite = *pPointRead;
+
+        /* scan curves in pPointRead */
+        for ( i2 = ldexp(1,n-1); i2 > 0; i2--)
+        {
+            *(pPointWrite+6) = *(pPointRead+3);
+            *(pPointWrite+1) = CenterPoint( * pPointRead,    *(pPointRead+1));
+            *(pPointWrite+3) = CenterPoint( *(pPointRead+1), *(pPointRead+2));
+            *(pPointWrite+5) = CenterPoint( *(pPointRead+2), *(pPointRead+3));
+            *(pPointWrite+2) = CenterPoint( *(pPointWrite+1), *(pPointWrite+3));
+            *(pPointWrite+4) = CenterPoint( *(pPointWrite+3), *(pPointWrite+5));
+            *(pPointWrite+3) = CenterPoint( *(pPointWrite+2), *(pPointWrite+4));
+
+            /* calculate the maximum deviation */
+            for ( i = 1; i <= 2; i++, pPointWrite += 3)
+            {
+                /* calculation of the curve with the functions
+                 * x(t) = ax*t^3 + bx*t^2 + cx*t + x0
+                 * y(t) = ay*t^3 + by*t^2 + cy*t + y0
+                 */
+                cx = 3*((pPointWrite+1)->x - pPointWrite->x);
+                bx = 3*((pPointWrite+2)->x - (pPointWrite+1)->x) - cx;
+                ax = (pPointWrite+3)->x - pPointWrite->x - bx - cx;
+                cy = 3*((pPointWrite+1)->y - pPointWrite->y);
+                by = 3*((pPointWrite+2)->y - (pPointWrite+1)->y) - cy;
+                ay = (pPointWrite+3)->y - pPointWrite->y - by - cy;
+                
+                /* calculation of the length of the single vector */
+                aLength[0] = sqrt(SqrDistPoints(pPointWrite[0], pPointWrite[1]));
+                aLength[1] = sqrt(SqrDistPoints(pPointWrite[1], pPointWrite[2]));
+                aLength[2] = sqrt(SqrDistPoints(pPointWrite[2], pPointWrite[3]));
+                
+                /* deviation for P1 */
+                t = aLength[0] + aLength[1] + aLength[2];
+                if (t == 0)
+                {   NSZoneFree(zone, apPointList[0]);
+                    NSZoneFree(zone, apPointList[1]);
+                    return 0;
+                }
+                t = aLength[0] / t;
+                curvePoint.x = ax*t*t*t + bx*t*t + cx*t + pPointWrite->x;
+                curvePoint.y = ay*t*t*t + by*t*t + cy*t + pPointWrite->y;
+                dist = SqrDistPoints(pPointWrite[1], curvePoint);
+                maxdist = (maxdist > dist) ? maxdist : dist;
+                
+                /* deviation for P2 */
+                t = aLength[0] + aLength[1] + aLength[2];
+                if (t == 0)
+                {   NSZoneFree(zone, apPointList[0]);
+                    NSZoneFree(zone, apPointList[1]);
+                    return 0;
+                }
+                t = (aLength[0] + aLength[1]) / t;
+                curvePoint.x = ax*t*t*t + bx*t*t + cx*t + pPointWrite->x;
+                curvePoint.y = ay*t*t*t + by*t*t + cy*t + pPointWrite->y;
+                dist = SqrDistPoints(pPointWrite[2], curvePoint);
+                maxdist = (maxdist > dist) ? maxdist : dist;
+            }
+            pPointRead  += 3;
+        }
+
+        apPointList[2] = apPointList[0];
+        apPointList[0] = apPointList[1];
+        apPointList[1] = apPointList[2];
+        n++;
+    } while ( maxdist > flatness*flatness);
+
+    NSZoneFree(zone, apPointList[1]);
+
+    *pts = apPointList[0];
+
+    /*{   NSPoint  pv0, pv1;
+
+        pPointRead = apPointList[0];
+        pv0 = *pPointRead++;
+        for (i = ldexp(3, n-1); i > 0; i--)
+        {   pv1 = *pPointRead++;
+            printf("%.1f %.1f ", pv1.x, pv1.y);
+            //[tmpStream appendLine:pv1.x :pv1.y];
+        }
+        printf("\n");
+        //free(apPointList[0]);
+    }*/
+
+    return ldexp(3, n-1) + 1;
+}
+
 BOOL vhfConvertCurveToArc(NSPoint pc[4], NSPoint *center, NSPoint *start, float *angle)
 {   NSPoint	pa, da, pb, db, ps[2];
     NSPoint	end = pc[3];
@@ -888,3 +1085,38 @@ NSRect vhfBoundsOfArc(NSPoint center, float radius, float begAngle, float angle)
 
     return bounds;
 }
+
+/* flatten arc into lines
+ * return: number of points in pts
+ *         pts has to be freed using NSZoneFree(NSDefaultMallocZone(), pts)
+ *
+ * modified: 2007-09-22
+ */
+int vhfConvertArcToLines(NSPoint center, float radius, float begAngle, float angle, float flatness, NSPoint **pts)
+{   double	chordAngle, dx = radius - flatness;  // we tune the flatness to make it smoother
+    int         n, i, nPts = 0;
+    NSPoint     startPoint, p, *ptsPtr;
+    NSZone      *zone = NSDefaultMallocZone();
+
+    startPoint = vhfPointRotatedAroundCenter(NSMakePoint(center.x+radius, center.y), begAngle, center);
+
+    if (dx <= 0.0)
+        n = 1;
+    else
+    {   chordAngle = RadToDeg(acos(dx/radius))*2.0;
+        chordAngle = Limit(chordAngle, 0.1, 5.0);
+        n = (int)ceil(Abs(angle) / chordAngle);         // number of lines
+    }
+
+    *pts = NSZoneMalloc(zone, (n+1) * sizeof(NSPoint));
+    ptsPtr = *pts;
+
+    chordAngle = angle / n;				// chord angle to get lines of equal length
+    for ( i=1; i<=n; i++ )
+    {   float	a = (double)i * chordAngle;
+
+        p = vhfPointRotatedAroundCenter(startPoint, a, center);
+        ptsPtr[nPts++] = p;
+    }
+    return nPts;
+}
