@@ -1,11 +1,11 @@
 /* GerberExportSub.m
  * Sub class of Gerber export
  *
- * Copyright (C) 2002-2005 by vhf interservice GmbH
+ * Copyright (C) 2002-2008 by vhf interservice GmbH
  * Author:   Ilonka Fleischmann
  *
  * created:  2002-04-25
- * modified: 2005-05-12
+ * modified: 2008-06-21 (bug fixes)
  *
  * This file is part of the vhf Export Library.
  *
@@ -65,8 +65,16 @@
 - (void)exportPolyLine:(VPolyLine*)g
 {   int		i, cnt = [(VPolyLine*)g numPoints];
 
+    // start polygon mode G36*
+    if ([g filled] && cnt > 3)
+        [self writePolygonMode:YES];
+
     for (i=0; i<cnt-1; i++)
         [self writeLine:[g pointWithNum:i] :[g pointWithNum:i+1]];
+
+    // end polygon mode G37*
+    if ([g filled] && cnt > 3)
+        [self writePolygonMode:NO];
 }
 
 - (void)exportRectangle:(VRectangle*)g
@@ -129,7 +137,7 @@
 }
 
 - (void)exportPath:(VPath*)path
-{   int			i, j, k, b, cnt=[[path list] count], subPathsCnt = 0;
+{   int			i, j, k, b, cnt=[[path list] count], subPathsCnt = 0, subPathsInCnt = 0;
     NSMutableArray	*subpaths = [NSMutableArray array], *subpathsIns = [NSMutableArray array];
     BOOL		lastDrawnIsClear = NO;
 
@@ -163,18 +171,31 @@
                     inside++;
             }
             [subpathsIns addObject:[NSNumber numberWithInt:inside]];
+            subPathsInCnt++;
         }
     }
 
     b = 0;
     while ( b >= 0 )
-    {   BOOL	drawed = NO;
+    {   BOOL	drawed = NO, somethingToDraw = NO;
 
-        if (!Even(b) && [path filled] && b && subPathsCnt > b)
-        {   [self writeLayerPolarityMode:YES]; // clear
+        for ( j=0; j<subPathsCnt; j++ )
+        {   int	inside;
+
+            inside = (subPathsInCnt > 1) ? [[subpathsIns objectAtIndex:j] intValue] : 0;
+            /* !b - we draw only the black one which have no realy no inside ; b - we draw only the white one */
+            if ( (inside == b) )
+            {   somethingToDraw  = YES;
+                break;
+            }
+        }
+
+        if (!Even(b) && [path filled] && b && subPathsCnt > b && somethingToDraw)
+        {
+            [self writeLayerPolarityMode:YES]; // clear
             lastDrawnIsClear = YES;
         }
-        else if (Even(b) && [path filled] && b && subPathsCnt > b)
+        else if (Even(b) && [path filled] && b && subPathsCnt > b && somethingToDraw)
         {   [self writeLayerPolarityMode:NO]; // dark
             lastDrawnIsClear = NO;
         }
@@ -183,7 +204,7 @@
         {   VPath	*sp = [subpaths objectAtIndex:j];
             int		cnt = [[sp list] count], inside;
 
-            inside = (subPathsCnt > 1) ? [[subpathsIns objectAtIndex:j] intValue] : 0;
+            inside = (subPathsInCnt > 1) ? [[subpathsIns objectAtIndex:j] intValue] : 0;
             /* !b - we draw only the black one which have no realy no inside ; b - we draw only the white one */
             if ( !(inside == b) )
                 continue;
@@ -243,10 +264,14 @@
     {
         /* count how many times each path is inside all others */
         for (j=0; j < polyCnt; j++)
-        {   id	gr = [[[[group list] objectAtIndex:polygonIs[j]] list] objectAtIndex:0];
-            NSPoint	pt = [gr pointWithNum:0];
-            int	inside = 0;
+        {   id      list = [[[group list] objectAtIndex:polygonIs[j]] list], gr;
+            NSPoint	pt;
+            int     inside = 0;
 
+            if (![list count])
+                continue;   // empty group
+            gr = [list objectAtIndex:0];
+            pt = [gr pointWithNum:0];
             for (k=0; k < polyCnt; k++)
             {   NSRect	kbounds = polyBounds[k];
 
@@ -407,146 +432,154 @@
     /* save */
 //    if ([fileManager isWritableFileAtPath:filename])
     {   id		layerList = [docView layerList];
-        int		i;
+        int		i, lcnt = [layerList count];
+        NSMutableArray	*gList = [NSMutableArray array];
 
 
 // 72 25400 div 72 25400 div scale\n
 
-        // first clip rect
-        for (i=(int)[layerList count]-1 ; i >=0  ; i--)
+        /* first clip rect */
+        for (i=0 ; i < lcnt  ; i++)
         {   LayerObject	*lObj = [layerList objectAtIndex:i];
 
-            if ([lObj type] == LAYER_CLIPPING || ![lObj state]) // visible - ????????????????????????
+            if ([lObj type] == LAYER_CLIPPING || ![lObj state]) // not visible - continue
                 continue;
-            {   int	j, k, cnt = [[lObj list] count];
-                int	insideIs[cnt], insideCnt = 0, polygonIs[cnt], polyCnt = 0, pathsIns[cnt];
-                NSRect	polyBounds[cnt];
 
-                /* get all bounds of polygons and */
-                /* get all indexes of polygons */
+            /* add all visible objects to gList */
+            {   int	j, cnt = [[lObj list] count];
+
                 for (j=0; j<cnt; j++)
-                {   id	g = [[lObj list] objectAtIndex:j];
+                    [gList addObject:[[lObj list] objectAtIndex:j]];
+            }
+        }
+        /* export gList */
+        {   int	j, k, cnt = [gList count];
+            int	insideIs[cnt], insideCnt = 0, polygonIs[cnt], polyCnt = 0, pathsIns[cnt];
+            NSRect	polyBounds[cnt];
 
-                    if ([g isKindOfClass:[VPath class]])
-                    {
-                        polygonIs[polyCnt] = j;
-                        polyBounds[polyCnt++] = [g bounds];
-                    }
-                }
+            /* get all bounds of polygons and */
+            /* get all indexes of polygons */
+            for (j=0; j<cnt; j++)
+            {   id	g = [gList objectAtIndex:j];
 
-                if (polyCnt > 1)
+                if ([g isKindOfClass:[VPath class]])
                 {
-                    /* count how many times each path is inside all others */
-                    for (j=0; j < polyCnt; j++)
-                    {   id	gr = [[[[lObj list] objectAtIndex:polygonIs[j]] list] objectAtIndex:0];
-                        NSPoint	pt = [gr pointWithNum:0];
-                        int	inside = 0;
-
-                        for (k=0; k < polyCnt; k++)
-                        {   NSRect	kbounds = polyBounds[k];
-
-                            if ( j == k )
-                                continue;
-                            if ( NSPointInRect(pt, kbounds) )
-                                inside++;
-                        }
-                        pathsIns[j] = inside;
-                    }
+                    polygonIs[polyCnt] = j;
+                    polyBounds[polyCnt++] = [g bounds];
                 }
+            }
 
-                /* get all indexes of graphics inside/intersect bounds of polygons */
-                for (j=0; j<cnt; j++)
-                {   id	g = [[lObj list] objectAtIndex:j];
+            if (polyCnt > 1)
+            {
+                /* count how many times each path is inside all others */
+                for (j=0; j < polyCnt; j++)
+                {   id	gr = [[[gList objectAtIndex:polygonIs[j]] list] objectAtIndex:0];
+                    NSPoint	pt = [gr pointWithNum:0];
+                    int	inside = 0;
 
-                    if (![g isKindOfClass:[VPath class]])
-                    {   NSRect	bounds = [g bounds];
+                    for (k=0; k < polyCnt; k++)
+                    {   NSRect	kbounds = polyBounds[k];
 
-                        for (k=0; k<polyCnt; k++)
-                        {
-                            if ( vhfIntersectsRect(polyBounds[k], bounds) )
-                            {   insideIs[insideCnt++] = j;
-                                break;
-                            }
-                        }
-                    }
-                }
-
-                /* draw graphics, but polygons, outside bounds of polygons */
-                /* draw polygons from outside to inside */
-                /* draw graphics inside bounds of polygons */
-                for (k=0; k < 3; k++)
-                {
-                    /* draw polygons from outside to inside */
-                    if ( k == 1 )
-                    {   int	b = 0;
-
-                        while ( b >= 0 )
-                        {   BOOL	drawed = NO;
-
-                            for (j=0; j<polyCnt; j++)
-                            {   int	inside;
-                                VPath	*sp = [[lObj list] objectAtIndex:polygonIs[j]];
-
-                                inside = (polyCnt > 1) ? pathsIns[j] : 0;
-                                /* b == inside; we draw from outside to inside */
-                                if ( !(inside == b) )
-                                    continue;
-
-                                drawed = YES;
-                                [self exportPath:sp];
-                            }
-                            if (drawed == NO)
-                                break;
-                            b++;
-                        }
-                        continue;
-                    }
-                    for (j=0; j<cnt; j++)
-                    {   id	g = [[lObj list] objectAtIndex:j];
-                        float	w = [g width];
-
-                        if ( !(!k && ![g isKindOfClass:[VPath class]] && !valueInArray(j, insideIs, insideCnt)) &&
-                             /*!(k == 1 && [g isKindOfClass:[VPath class]]) &&*/
-                             !(k == 2 && ![g isKindOfClass:[VPath class]] && valueInArray(j, insideIs, insideCnt)) )
+                        if ( j == k )
                             continue;
+                        if ( NSPointInRect(pt, kbounds) )
+                            inside++;
+                    }
+                    pathsIns[j] = inside;
+                }
+            }
 
-                        if ([g isKindOfClass:[VRectangle class]] && [g filled] && ![(VRectangle*)g radius])
-                        {   NSSize	rsize = [(VRectangle*)g size];
+            /* get all indexes of graphics inside/intersect bounds of polygons */
+            for (j=0; j<cnt; j++)
+            {   id	g = [gList objectAtIndex:j];
 
-                            [self writeRectangleTool:w+rsize.width :w+rsize.height];
+                if (![g isKindOfClass:[VPath class]])
+                {   NSRect	bounds = [g bounds];
+
+                    for (k=0; k<polyCnt; k++)
+                    {
+                        if ( vhfIntersectsRect(polyBounds[k], bounds) )
+                        {   insideIs[insideCnt++] = j;
+                            break;
                         }
-                        else if (w ||
-                                 ([g isKindOfClass:[VArc class]] && Abs([(VArc*)g angle]) == 360.0 && [(VArc*)g filled]))
-                        {
-                            if ([g isKindOfClass:[VArc class]] && Abs([(VArc*)g angle]) == 360.0 && [(VArc*)g filled])
-                                w += [(VArc*)g radius]*2.0;
-                            [self writeCircleTool:w];
-                        }
-
-                        if ( [g isKindOfClass:[VLine class]] )
-                            [self writeLine:[g pointWithNum:0] :[g pointWithNum:1]];
-                        else if ( [g isKindOfClass:[VPolyLine class]] )
-                            [self exportPolyLine:g];
-                        else if (  [g isKindOfClass:[VRectangle class]] )
-                            [self exportRectangle:g];
-                        else if (  [g isKindOfClass:[VArc class]] )
-                            [self exportArc:g];
-                        else if (  [g isKindOfClass:[VCurve class]] )
-                            [self exportCurve:g];
-                        else if (  [g isKindOfClass:[VPath class]] )
-                            NSLog(@"GerberExportSub.m: exportToFile, exportPath\n");/*[self exportPath:g];*/
-                        else if (  [g isKindOfClass:[VGroup class]] )
-                            [self exportGroup:g];
-                        else if (  [g isKindOfClass:[VText class]] )
-                            [self exportText:g];
                     }
                 }
             }
+
+            /* draw graphics, but polygons, outside bounds of polygons */
+            /* draw polygons from outside to inside */
+            /* draw graphics inside bounds of polygons */
+            for (k=0; k < 3; k++)
+            {
+                /* draw polygons from outside to inside */
+                if ( k == 1 )
+                {   int	b = 0;
+
+                    while ( b >= 0 )
+                    {   BOOL	drawed = NO;
+
+                        for (j=0; j<polyCnt; j++)
+                        {   int	inside;
+                            VPath	*sp = [gList objectAtIndex:polygonIs[j]];
+
+                            inside = (polyCnt > 1) ? pathsIns[j] : 0;
+                            /* b == inside; we draw from outside to inside */
+                            if ( !(inside == b) )
+                                continue;
+
+                            drawed = YES;
+                            [self exportPath:sp];
+                        }
+                        if (drawed == NO)
+                            break;
+                        b++;
+                    }
+                    continue;
+                }
+                for (j=0; j<cnt; j++)
+                {   id	g = [gList objectAtIndex:j];
+                    float	w = [g width];
+
+                    if ( !(!k && ![g isKindOfClass:[VPath class]] && !valueInArray(j, insideIs, insideCnt)) &&
+                         /*!(k == 1 && [g isKindOfClass:[VPath class]]) &&*/
+                         !(k == 2 && ![g isKindOfClass:[VPath class]] && valueInArray(j, insideIs, insideCnt)) )
+                        continue;
+
+                    if ([g isKindOfClass:[VRectangle class]] && [g filled] && ![(VRectangle*)g radius])
+                    {   NSSize	rsize = [(VRectangle*)g size];
+
+                        [self writeRectangleTool:w+rsize.width :w+rsize.height];
+                    }
+                    else if (w ||
+                             ([g isKindOfClass:[VArc class]] && Abs([(VArc*)g angle]) == 360.0 && [(VArc*)g filled]))
+                    {
+                        if ([g isKindOfClass:[VArc class]] && Abs([(VArc*)g angle]) == 360.0 && [(VArc*)g filled])
+                            w += [(VArc*)g radius]*2.0;
+                        [self writeCircleTool:w];
+                    }
+
+                    if ( [g isKindOfClass:[VLine class]] )
+                        [self writeLine:[g pointWithNum:0] :[g pointWithNum:1]];
+                    else if ( [g isKindOfClass:[VPolyLine class]] )
+                        [self exportPolyLine:g];
+                    else if (  [g isKindOfClass:[VRectangle class]] )
+                        [self exportRectangle:g];
+                    else if (  [g isKindOfClass:[VArc class]] )
+                        [self exportArc:g];
+                    else if (  [g isKindOfClass:[VCurve class]] )
+                        [self exportCurve:g];
+                    else if (  [g isKindOfClass:[VPath class]] )
+                        NSLog(@"GerberExportSub.m: exportToFile, exportPath\n");/*[self exportPath:g];*/
+                              else if (  [g isKindOfClass:[VGroup class]] )
+                              [self exportGroup:g];
+                              else if (  [g isKindOfClass:[VText class]] )
+                              [self exportText:g];
+                }
+            }
         }
-        savedOk = [self saveToFile:filename];
+        [gList removeAllObjects];
     }
-//    else
-//        NSRunAlertPanel(SAVE_TITLE, DIR_NOT_WRITABLE, nil, nil, nil);
+    savedOk = [self saveToFile:filename];
 
     /* restore backup */
     if (!savedOk)
