@@ -1,11 +1,11 @@
 /* dvPasteboard.m
  * Pasteboard additions for Cenon DocView class
  *
- * Copyright (C) 1997-2006 by vhf interservice GmbH
+ * Copyright (C) 1997-2008 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1997
- * modified: 2006-12-13
+ * modified: 2008-10-21 (pasting of several layers to one layer keeps order of graphics elements)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -384,8 +384,8 @@ NSArray *e2TypesCenonExports(void)
 - copyToPasteboard:(NSPasteboard *)pboard types:(NSArray *)typesList
 {
     if ([slayList count])
-    {   NSMutableArray	*types = [[[NSMutableArray alloc] initWithCapacity:1] autorelease];
-        NSData		*data;
+    {   NSMutableArray  *types = [[[NSMutableArray alloc] initWithCapacity:1] autorelease];
+        NSData          *data;
 
         [types addObject:e2PboardType];
         if (!typesList || e2IncludesType(typesList, NSPostScriptPboardType))
@@ -442,11 +442,11 @@ NSArray *e2TypesCenonExports(void)
  * by the caller).
  */
 - (NSArray *)pasteFromPasteboard:(NSPasteboard *)pboard andLink:(LinkType)doLink at:(const NSPoint *)center
-{   int		i, l;
-    id		change;
-    NSArray	*pblist = nil;
-    VGraphic	*graphic = nil;
-    BOOL	pasteDrawType = NO;
+{   int         i, l;
+    id          change;
+    NSArray     *pblist = nil;
+    VGraphic    *graphic = nil;
+    BOOL        pasteDrawType = NO;
 
     //if (!linkManager)
         doLink = DontLink;
@@ -463,7 +463,7 @@ NSArray *e2TypesCenonExports(void)
             {   NSPoint	offset = NSMakePoint(0, 0);
                 NSRect	pbBounds, bounds = [self bounds];
 
-		pblist = [[[NSMutableArray alloc] initWithArray:[unarchiver decodeObject]] autorelease];
+                pblist = [[[NSMutableArray alloc] initWithArray:[unarchiver decodeObject]] autorelease];
 
                 /* keep graphics inside bounds (consecutive paste) or visible rect (1st paste) */
                 pbBounds = [self boundsOfArray:pblist withKnobs:NO];
@@ -508,15 +508,16 @@ NSArray *e2TypesCenonExports(void)
                             {   fixLayer = l; break; }
                         }
                     }
-                    cnt = (fixLayer<0) ? Min([layerList count], [pblist count]) : [pblist count] ;
-                    for ( l=cnt-1; l>=0; l-- )
-                    {   int		insertLayer = (fixLayer>=0) ? fixLayer : l;
-                        NSMutableArray	*slist;
-                        LayerObject	*layer;
-                        NSMutableArray	*srcList = [pblist objectAtIndex:l];
+                    cnt = (fixLayer < 0) ? Min([layerList count], [pblist count]) : [pblist count];
+                    //for ( l=cnt-1; l>=0; l-- )
+                    for ( l=0; l < cnt; l++ )
+                    {   int             insertLayer = (fixLayer >= 0) ? fixLayer : l;
+                        NSMutableArray  *slist;
+                        LayerObject     *layer;
+                        NSMutableArray  *srcList = [pblist objectAtIndex:l];
 
                         /* paste to 1st editable layer */
-                        if ( fixLayer<0 && ![[layerList objectAtIndex:l] editable])
+                        if ( fixLayer < 0 && ![[layerList objectAtIndex:l] editable])
                         {
                             for ( i=0; i<cnt; i++ )
                             {
